# CityDoctor2 Extensions
Extension modules to the core functionality of CityDoctor2. 

# Loading

Except for CityDoctorGUI, all extension modules are not loaded into the Maven project by default.
If you wish to load an extension, you can either import the respective Maven module directly, or adjust the CityDoctorParent
pom.xml and move the respective modules out of the comment-block in the module list found at the end of the file.

# Building

All loaded modules will also be built upon building of the CityDoctorParent.

# Module Information

## CityDoctorAutoPro

![Maintenance Level: Deprecated](../resources/Badges/MaintenanceLevel_Deprecated.svg)

The AutoPro (short for automated processing) module is an interface for the conversion of the internal data model to a C++
representation, to allow use of advanced functionalities of CGAL with CityDoctor2.

### Usage

The AutoPro module is deprecated and will be removed in a future release, once the C++ functionalities have been ported to Java.

AutoPro requires a collection of .dll files to run, some of which are not available as open source projects. Access to the collection is only granted upon request. Contact a Maintainer for further information.

## CityDoctorGUI
![Maintenance Level: Actively Maintained](../resources/Badges/MaintenanceLevel_ActivelyMaintained.svg)

The GUI module offers a graphical user interface for CityDoctor2, including a model viewer allowing for visual
inspection of validation results and found errors.

### View Plugins

The GUI is expandable via plugins, enabling the creation of views for custom functionalities or extensions of the CityDoctor2 core.  
Custom views can be created by extending the Abstract class `View` of the CityDoctorGUI and need to be registered by calling
`ViewRegistration.registerView(View view);` before starting the GUI.

### Usage

> ⚠ CityDoctorGUI requires a JDK/JRE containing the JavaFX library to run.  

Prebuilt binaries for the GUI are available in [CityDoctorReleases](https://transfer.hft-stuttgart.de/gitlab/citydoctor/citydoctorreleases).
After extracting the archive the GUI can be started with **start.bat**.

Binaries can be built with Maven using the create-binaries profile from the CityDoctorParent directory:  
```
mvn install -P create-binaries
```   

Lastly, the GUI can also be started by calling `MainWindow.main(args[])`.


## CityDoctorHealer
![Maintenance Level: Inactively Maintained](../resources/Badges/MaintenanceLevel_InactivelyMaintained.svg)

CityDoctorHealer implements functionalities for the automated repair of geometries containing errors.
The healer uses naive approaches to fix geometrical errors, and thus, it cannot be guaranteed that semantical
correctness is maintained after the healing process.  


### Usage

Usage of CityDoctorHealer is similar to [CityDoctor](https://transfer.hft-stuttgart.de/gitlab/citydoctor/citydoctor2/-/tree/dev_GUI#usage).
An example start command looks like this:
```
java -classpath libs/*;plugins/*;CityDoctorHealer-<version>.jar de.hft.stuttgart.citydoctor2.CityDoctorHealer -in <path-to-gml-file>.gml -config <path-to-validation-config>.yml -xmlReport <path-to-xml-output>.xml -pdfReport <path-to-pdf-output>.pdf -out <path-to-output-gml>.gml
```
Note: Unlike the CityDoctor core, the _-out_ argument is required

To prevent infinite loops the `Healer` class has a limit for the number of healing iterations (default: 200). This limit
can be changed by calling `Healer.setNumberOfIterations(int limit)`. 

## CityDoctorHealerGenetic
![Maintenance Level: Abandoned](../resources/Badges/MaintenanceLevel_Abandoned.svg)

CityDoctorHealerGenetic uses a genetical algorithm to establish an optimal CityDoctorHealer healing-plan for geometries containing errors.

### Usage
> ⚠ This module has been abandoned. As of writing the module is still functional, but it may break at any time.
>Issues, bug reports and suggestions for this module will not be processed. Use at your own discretion.

Usage is nearly identical to CityDoctorHealer, only requiring changing the classpath to the HealerGenetic jar.

A View for usage with CityDoctorGUI is included.

## CityDoctorHealerGUI
![Maintenance Level: Inactively Maintained](../resources/Badges/MaintenanceLevel_InactivelyMaintained.svg)

CityDoctorGUI-view for CityDoctorHealer.

### Usage

See [CityDoctorGUI](#view-plugins)


## CityDoctorWebService
![Maintenance Level: Abandoned](../resources/Badges/MaintenanceLevel_Abandoned.svg)

Web service API for CityDoctor.

### Usage
> ⚠ This module has been abandoned. As of writing the module is still functional, but it may break at any time.
>Issues, bug reports and suggestions for this module will not be processed. Use at your own discretion.

Run on a Tomcat Server.


# Build

Use [Maven](https://maven.apache.org/) to build CityDoctor2.

```bash
cd CityDoctorParent
mvn install
```


## License
[LGPL](http://www.gnu.org/licenses/lgpl-3.0.de.html)
